<?php

declare(strict_types=1);

namespace Drupal\field_visibility_conditions\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure field visibility conditions settings.
 */
class FieldVisibilityConditionsSettingsForm extends ConfigFormBase {

  /**
   * Config settings name.
   *
   * @var string
   */
  public const SETTINGS = 'field_visibility_conditions.settings';

  /**
   * The condition manager.
   *
   * @var \Drupal\Core\Condition\ConditionManager
   */
  protected $conditionManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('plugin.manager.condition')
    );
  }

  /**
   * Constructor for the settings form.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config_manager
   *   The typed config manager.
   * @param \Drupal\Core\Condition\ConditionManager $condition_manager
   *   The condition plugin manager.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typed_config_manager,
    ConditionManager $condition_manager,
  ) {
    parent::__construct($config_factory, $typed_config_manager);
    $this->conditionManager = $condition_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'field_visibility_conditions_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [static::SETTINGS];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config(static::SETTINGS);
    $enabled_conditions = $config->get('enabled_conditions') ?: [];

    // Get all available conditions.
    $available_conditions = $this->conditionManager->getDefinitions();

    $form['conditions'] = [
      '#type' => 'details',
      '#title' => $this->t('Available Conditions'),
      '#description' => $this->t('Select which conditions should be available for field visibility.'),
      '#open' => TRUE,
    ];

    $form['conditions']['enabled_conditions'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enabled conditions'),
      '#options' => array_map(function ($condition) {
        return $condition['label'];
      }, $available_conditions),
      '#default_value' => $enabled_conditions,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $values = $form_state->getValues();

    // Filter out unselected conditions.
    $enabled_conditions = array_filter($values['enabled_conditions']);

    $this->config(static::SETTINGS)
      ->set('enabled_conditions', $enabled_conditions)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
