<?php

declare(strict_types=1);

namespace Drupal\Tests\field_visibility_conditions\Functional;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Tests\field_ui\Traits\FieldUiTestTrait;
use Drupal\Tests\BrowserTestBase;
use Drupal\Core\Url;

/**
 * Tests the field visibility conditions settings form.
 *
 * @group field_visibility_conditions
 */
class FieldVisibilityConditionsSettingsTest extends BrowserTestBase {
  use StringTranslationTrait;
  use FieldUiTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field_visibility_conditions',
    'field_visibility_conditions_test',
    'field_ui',
    'node',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The admin user that will be created.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * A test content type.
   *
   * @var \Drupal\node\NodeTypeInterface
   */
  protected $contentType;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a content type for testing.
    $this->contentType = $this->drupalCreateContentType();

    // Create a user with permissions to manage fields and field settings.
    $this->adminUser = $this->drupalCreateUser([
      'administer field visibility conditions',
      'administer content types',
      'administer node fields',
    ]);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests the admin settings form and its effect on field configuration.
   */
  public function testConditionFiltering(): void {
    // Visit the settings form.
    $this->drupalGet('admin/config/content/field-visibility-conditions');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Field Visibility Conditions Settings');

    // Check the form exists.
    $this->assertSession()->elementExists('css', 'form.field-visibility-conditions-settings');

    // Find all checkboxes and their names.
    $page = $this->getSession()->getPage();
    $checkboxes = $page->findAll('css', 'input[type="checkbox"]');

    // Find at least one checkbox to enable.
    $edit = [];
    $userRoleFound = FALSE;

    foreach ($checkboxes as $checkbox) {
      $name = $checkbox->getAttribute('name');
      $value = $checkbox->getAttribute('value');

      // Also enable user_role if found.
      if (strpos($name, 'user_role') !== FALSE) {
        $edit[$name] = $value;
        $userRoleFound = TRUE;
      }
    }

    // If we found checkboxes to enable, submit the form.
    $this->assertNotEmpty($edit, 'No condition checkboxes found on the form.');
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Create a field to test the conditions.
    $this->drupalGet('admin/structure/types/manage/' . $this->contentType->id() . '/fields/add-field');
    $edit = [
      'new_storage_type' => 'plain_text',
    ];
    $this->submitForm($edit, 'Continue');

    $edit = [
      'group_field_options_wrapper' => 'string',
      'label' => 'Test Field',
      'field_name' => 'test_field',
    ];
    $this->submitForm($edit, 'Continue');

    $edit = [
      'label' => 'Test Field',
      'field_storage[subform][settings][max_length]' => 255,
    ];
    $this->submitForm($edit, 'Save settings');

    // Go to the field settings form.
    $this->drupalGet('admin/structure/types/manage/' . $this->contentType->id() . '/fields/node.' . $this->contentType->id() . '.field_test_field');

    // Check that the Field Visibility section exists.
    $this->assertSession()->pageTextContains('Field Visibility');

    // If we enabled user_role, we should see Role in the form.
    if ($userRoleFound) {
      $this->assertSession()->pageTextContains('Role');
    }
  }

  /**
   * Tests the settings form.
   */
  public function testSettingsForm(): void {
    // Navigate to the admin form.
    $this->drupalGet(Url::fromRoute('field_visibility_conditions.settings'));
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Field Visibility Conditions Settings');

    // Test that the form contains checkboxes for conditions including our test
    // one.
    $this->assertSession()->fieldExists('enabled_conditions[request_path]');
    $this->assertSession()->fieldExists('enabled_conditions[user_role]');
    $this->assertSession()->fieldExists('enabled_conditions[test_condition]');

    // Select some conditions and submit the form.
    $edit = [
      'enabled_conditions[user_role]' => 'user_role',
      'enabled_conditions[test_condition]' => 'test_condition',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Verify that the settings were saved correctly by checking config.
    $config = $this->config('field_visibility_conditions.settings');
    $enabled_conditions = $config->get('enabled_conditions');
    $this->assertEquals('user_role', $enabled_conditions['user_role']);
    $this->assertEquals('test_condition', $enabled_conditions['test_condition']);
    $this->assertArrayNotHasKey('request_path', $enabled_conditions);

    // Create a test field to verify the conditions appear in field settings.
    $field_name = 'test_field';
    $field_storage_edit = [];
    $field_edit = [];
    $this->fieldUIAddNewField(
      'admin/structure/types/manage/' . $this->contentType->id(),
      $field_name,
      'Test field',
      'string',
      $field_storage_edit,
      $field_edit
    );

    // Go to edit the field configuration.
    $this->drupalGet('admin/structure/types/manage/' . $this->contentType->id() . '/fields/node.' . $this->contentType->id() . '.field_' . $field_name);
    $this->assertSession()->statusCodeEquals(200);

    // Verify that the enabled conditions appear in the field settings.
    $this->assertSession()->pageTextContains('Field Visibility');
    $this->assertSession()->pageTextContains('User role');
    $this->assertSession()->pageTextContains('Test condition');

    // Verify that disabled conditions do not appear.
    $this->assertSession()->pageTextNotContains('Request path');

    // Now update settings to disable a condition.
    $this->drupalGet(Url::fromRoute('field_visibility_conditions.settings'));
    $edit = [
      'enabled_conditions[user_role]' => 'user_role',
      'enabled_conditions[test_condition]' => FALSE,
    ];
    $this->submitForm($edit, 'Save configuration');

    // Check our field form again - test condition should be gone.
    $this->drupalGet('admin/structure/types/manage/' . $this->contentType->id() . '/fields/node.' . $this->contentType->id() . '.field_' . $field_name);
    $this->assertSession()->pageTextContains('Field Visibility');
    $this->assertSession()->pageTextContains('User role');
    $this->assertSession()->pageTextNotContains('Test condition');
  }

}
