<?php

declare(strict_types=1);

namespace Drupal\Tests\field_visibility_conditions\Kernel;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormState;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\NodeType;
use Drupal\Tests\field_visibility_conditions\Traits\TestConditionTrait;
use Drupal\Tests\node\Traits\ContentTypeCreationTrait;
use Drupal\user\Entity\User;

/**
 * Tests the field visibility conditions functionality.
 *
 * @group field_visibility_conditions
 */
class FieldVisibilityConditionsTest extends KernelTestBase {

  use ContentTypeCreationTrait;
  use TestConditionTrait;

  /**
   * The modules to enable for this test.
   *
   * @var array
   */
  protected static $modules = [
    'field',
    'field_visibility_conditions',
    'field_visibility_conditions_test',
    'field_ui',
    'filter',
    'node',
    'path_alias',
    'system',
    'text',
    'user',
  ];

  /**
   * The node type to test with.
   *
   * @var \Drupal\node\Entity\NodeType
   */
  protected ?NodeType $nodeType;

  /**
   * The test field.
   *
   * @var \Drupal\field\Entity\FieldConfig
   */
  protected ?FieldConfig $field;

  /**
   * The test user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected ?User $user;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installEntitySchema('path_alias');
    $this->installConfig(['field', 'node', 'system', 'user']);

    // Create a test user.
    $this->user = User::create([
      'name' => 'test_user',
      'mail' => 'test@example.com',
      'status' => 1,
    ]);
    $this->user->save();

    // Set the current user.
    $this->container->get('current_user')->setAccount($this->user);

    // Load or create the content type.
    $this->nodeType = NodeType::load('test_article');
    if (!$this->nodeType) {
      $this->nodeType = $this->createContentType(['type' => 'test_article']);
    }

    // Create a test field if it doesn't exist.
    $field_storage = FieldStorageConfig::loadByName('node', 'field_test');
    if (!$field_storage) {
      $field_storage = FieldStorageConfig::create([
        'field_name' => 'field_test',
        'entity_type' => 'node',
        'type' => 'text',
        'cardinality' => FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED,
      ]);
      $field_storage->save();
    }

    // Load or create the field config.
    $this->field = FieldConfig::loadByName('node', 'test_article', 'field_test');
    if (!$this->field) {
      $this->field = FieldConfig::create([
        'field_storage' => $field_storage,
        'bundle' => 'test_article',
        'label' => 'Test field',
      ]);
      $this->field->save();
    }

    // Load or create the form display.
    $form_display = EntityFormDisplay::load('node.test_article.default');
    if (!$form_display) {
      $form_display = EntityFormDisplay::create([
        'targetEntityType' => 'node',
        'bundle' => 'test_article',
        'mode' => 'default',
        'status' => TRUE,
      ]);
    }
    $form_display->setComponent('field_test', [
      'type' => 'text_textfield',
      'weight' => 0,
    ])->save();
  }

  /**
   * {@inheritdoc}
   */
  protected function tearDown(): void {
    // Clean up the form display after each test.
    if ($form_display = EntityFormDisplay::load('node.test_article.default')) {
      $form_display->delete();
    }
    parent::tearDown();
  }

  /**
   * Tests that conditions are properly added to the field config form.
   */
  public function testFieldConfigFormAlter(): void {
    $form = [];

    // Get the form using the entity form builder.
    $form = $this->container->get('entity.form_builder')
      ->getForm($this->field, 'default', []);

    // Assert the conditions section exists.
    $this->assertArrayHasKey('third_party_settings', $form);
    $this->assertArrayHasKey('field_visibility_conditions', $form['third_party_settings']);
    $this->assertEquals('Field Visibility', (string) $form['third_party_settings']['field_visibility_conditions']['#title']);

    // Assert our test condition is present.
    $this->assertArrayHasKey('test_condition', $form['third_party_settings']['field_visibility_conditions']);
  }

  /**
   * Tests that conditions properly control field visibility.
   */
  public function testFieldVisibilityConditions(): void {
    // Configure the test field with our test condition.
    $this->field->setThirdPartySetting('field_visibility_conditions', 'test_condition', [
      'id' => 'test_condition',
      'condition_met' => TRUE,
      'negate' => FALSE,
    ]);
    $this->field->save();

    $this->container->get('cache_tags.invalidator')->invalidateTags(['field_config']);

    // Create a node form.
    $node = $this->container->get('entity_type.manager')
      ->getStorage('node')
      ->create([
        'type' => 'test_article',
        'title' => 'Test node',
      ]);
    $node->save();

    // Create a form state and form object.
    $form_state = new FormState();
    $form_object = $this->container->get('entity_type.manager')
      ->getFormObject('node', 'default');
    $form_object->setEntity($node);
    $form_state->setFormObject($form_object);

    // Get the form.
    $form = $this->container->get('form_builder')->buildForm($form_object, $form_state);

    // Assert the field is visible when condition is met.
    $this->assertTrue($form['field_test']['#access']);

    // Change the condition to not be met.
    $this->field->setThirdPartySetting('field_visibility_conditions', 'test_condition', [
      'id' => 'test_condition',
      'condition_met' => FALSE,
      'negate' => FALSE,
    ]);
    $this->field->save();

    // Clear all caches to ensure the new condition is picked up.
    $this->container->get('cache_tags.invalidator')->invalidateTags(['field_config']);

    // Rebuild the form.
    $fresh_node = $this->container->get('entity_type.manager')
      ->getStorage('node')
      ->load($node->id());
    $form_object = $this->container->get('entity_type.manager')
      ->getFormObject('node', 'default');
    $form_object->setEntity($fresh_node);
    $form_state->setFormObject($form_object);
    $form = $this->container->get('form_builder')->buildForm($form_object, $form_state);

    // Assert the field is hidden when condition is not met.
    $this->assertFalse($form['field_test']['#access']);
  }

  /**
   * Tests that multiple conditions work together properly.
   */
  public function testMultipleConditions(): void {
    // Configure the test field with two test conditions.
    $this->field->setThirdPartySetting('field_visibility_conditions', 'test_condition', [
      'id' => 'test_condition',
      'condition_met' => TRUE,
      'negate' => FALSE,
    ]);
    $this->field->setThirdPartySetting('field_visibility_conditions', 'test_condition_two', [
      'id' => 'test_condition_two',
      'condition_met' => TRUE,
      'negate' => FALSE,
    ]);
    $this->field->save();
  }

}
